diff --git a/win32/SciTEWin.h b/win32/SciTEWin.h
--- a/win32/SciTEWin.h
+++ b/win32/SciTEWin.h
@@ -201,6 +201,7 @@
 
 	// Tab Bar
 	HFONT fontTabs;
+	std::vector<GUI::gui_string> tabNamesCurrent;
 
 	/// Preserve focus during deactivation
 	HWND wFocus;
@@ -295,6 +296,7 @@
 	void SetFileProperties(PropSetFile &ps) override;
 	void SetStatusBarText(const char *s) override;
 
+	void UpdateTabs(const std::vector<GUI::gui_string> &tabNames) override;
 	void TabInsert(int index, const GUI::gui_char *title) override;
 	void TabSelect(int index) override;
 	void RemoveAllTabs() override;
diff --git a/win32/SciTEWinBar.cxx b/win32/SciTEWinBar.cxx
--- a/win32/SciTEWinBar.cxx
+++ b/win32/SciTEWinBar.cxx
@@ -74,6 +74,38 @@
 		      SB_SETTEXT, 0, reinterpret_cast<LPARAM>(barText.c_str()));
 }
 
+void SciTEWin::UpdateTabs(const std::vector<GUI::gui_string> &tabNames) {
+	if (tabNames == tabNamesCurrent) {
+		return;
+	}
+	//::SendMessage(HwndOf(wTabBar), WM_SETREDRAW, 0, 0);
+	while (tabNames.size() < tabNamesCurrent.size()) {
+		TabCtrl_DeleteItem(HwndOf(wTabBar), tabNamesCurrent.size()-1);
+		tabNamesCurrent.pop_back();
+	}
+	while (tabNames.size() > tabNamesCurrent.size()) {
+		GUI::gui_string tabNameNext = tabNames[tabNamesCurrent.size()];
+		TCITEMW tie{};
+		tie.mask = TCIF_TEXT | TCIF_IMAGE;
+		tie.iImage = -1;
+		tie.pszText = tabNameNext.data();
+		TabCtrl_InsertItem(HwndOf(wTabBar), tabNamesCurrent.size(), &tie);
+		tabNamesCurrent.push_back(tabNameNext);
+	}
+	for (size_t tab = 0; tab < tabNames.size(); tab++) {
+		if (tabNames[tab] != tabNamesCurrent[tab]) {
+			TCITEMW tie {};
+			tie.mask = TCIF_TEXT | TCIF_IMAGE;
+			tie.iImage = -1;
+			GUI::gui_string tabNameCopy(tabNames[tab]);
+			tie.pszText = tabNameCopy.data();
+			TabCtrl_SetItem(HwndOf(wTabBar), tab, &tie);
+		}
+	}
+	//::SendMessage(HwndOf(wTabBar), WM_SETREDRAW, 1, 0);
+	tabNamesCurrent = tabNames;
+}
+
 void SciTEWin::TabInsert(int index, const GUI::gui_char *title) {
 	TCITEMW tie;
 	tie.mask = TCIF_TEXT | TCIF_IMAGE;
@@ -84,7 +116,9 @@
 }
 
 void SciTEWin::TabSelect(int index) {
-	TabCtrl_SetCurSel(HwndOf(wTabBar), index);
+	if (index != TabCtrl_GetCurSel(HwndOf(wTabBar))) {
+		TabCtrl_SetCurSel(HwndOf(wTabBar), index);
+	}
 }
 
 void SciTEWin::RemoveAllTabs() {
diff --git a/src/SciTEBuffers.cxx b/src/SciTEBuffers.cxx
--- a/src/SciTEBuffers.cxx
+++ b/src/SciTEBuffers.cxx
@@ -1117,6 +1117,13 @@
 	buffers.CommitStackSelection();
 }
 
+void SciTEBase::UpdateTabs(const std::vector<GUI::gui_string> &tabNames) {
+	RemoveAllTabs();
+	for (int t = 0; t < static_cast<int>(tabNames.size()); t++) {
+		TabInsert(t, tabNames[t].c_str());
+	}
+}
+
 static void EscapeFilePathsForMenu(GUI::gui_string &path) {
 	// Escape '&' characters in path, since they are interpreted in
 	// menus.
@@ -1133,7 +1140,8 @@
 	if (buffers.size() <= 1) {
 		DestroyMenuItem(menuBuffers, IDM_BUFFERSEP);
 	}
-	RemoveAllTabs();
+
+	std::vector<GUI::gui_string> tabNames;
 
 	int pos;
 	for (pos = buffers.lengthVisible; pos < bufferMax; pos++) {
@@ -1194,6 +1202,6 @@
 			}
 
 			SetMenuItem(menuBuffers, menuStart + pos + 1, itemID, entry.c_str());
-			TabInsert(pos, titleTab.c_str());
+			tabNames.push_back(titleTab);
 		}
 	}
@@ -1198,5 +1206,7 @@
 		}
 	}
+	UpdateTabs(tabNames);
+
 	CheckMenus();
 #if !defined(GTK)
 
diff --git a/src/SciTEBase.h b/src/SciTEBase.h
--- a/src/SciTEBase.h
+++ b/src/SciTEBase.h
@@ -578,6 +578,7 @@
 	void PrevInStack();
 	void EndStackedTabbing();
 
+	virtual void UpdateTabs(const std::vector<GUI::gui_string> &tabNames);
 	virtual void TabInsert(int index, const GUI::gui_char *title) = 0;
 	virtual void TabSelect(int index) = 0;
 	virtual void RemoveAllTabs() = 0;
